@@ -37,15 +37,41 @@ Simple example:
 """
 
 import datetime
-from decimal import Decimal, InvalidOperation
+import gettext
 import locale
+import re
 import sys
 import time
 
 from kiwi import ValueUnset
+from kiwi.enums import Alignment
+
+try:
+    from decimal import Decimal, InvalidOperation
+    HAVE_DECIMAL = True
+    Decimal, InvalidOperation # pyflakes
+except:
+    HAVE_DECIMAL = False
+    class Decimal(float):
+        pass
+    InvalidOperation = ValueError
+
+if sys.platform == 'win32':
+    try:
+        import ctypes
+        def GetLocaleInfo(value):
+            s = ctypes.create_string_buffer("\000" * 255)
+            ctypes.windll.kernel32.GetLocaleInfoA(0, value, s, 255)
+            return str(s.value)
+    except ImportError:
+        def GetLocaleInfo(value):
+            raise Exception(
+                "ctypes is required for datetime types on win32")
 
 __all__ = ['ValidationError', 'lformat', 'converter', 'format_price']
 
+_ = lambda m: gettext.dgettext('kiwi', m)
+
 number = (int, float, long, Decimal)
 
 class ValidationError(Exception):
@@ -64,14 +90,48 @@ class ConverterRegistry:
             raise TypeError("converter_type must be a BaseConverter subclass")
 
         c = converter_type()
+        if c.type in self._converters:
+            raise ValueError(converter_type)
         self._converters[c.type] = c
 
+    def remove(self, converter_type):
+        """
+        Removes converter_type from the registry
+        @param converter_type: a L{BaseConverter} subclass
+        """
+        if not issubclass(converter_type, BaseConverter):
+            raise TypeError("converter_type must be a BaseConverter subclass")
+
+        ctype = converter_type.type
+        if not ctype in self._converters:
+            raise KeyError(converter_type)
+
+        del self._converters[ctype]
+
     def get_converter(self, converter_type):
         try:
             return self._converters[converter_type]
         except KeyError:
             raise KeyError(converter_type)
 
+    def get_converters(self, base_classes=None):
+        if base_classes is None:
+            return self._converters.values()
+        
+        converters = []
+        if object in base_classes:
+            #: Ugly, but cannot remove from tuple!
+            base_classes = list(base_classes)
+            base_classes.remove(object)
+            base_classes = tuple(base_classes)
+            converters.append(self._converters[object])
+
+        for datatype in self._converters:
+            if issubclass(datatype, base_classes):
+                converters.append(self._converters[datatype])
+
+        return converters
+
     def check_supported(self, data_type):
         value = None
         for t in self._converters.values():
@@ -129,8 +189,13 @@ class BaseConverter(object):
     """
     Abstract converter used by all datatypes
     @cvar type:
+    @cvar name: The name of the datatype.
+    @cvar align: The alignment of the datatype. Normally right for numbers
+                 and dates, left for others. Default is left.
     """
     type = None
+    name = None
+    align = Alignment.LEFT
 
     def get_compare_function(self):
         """
@@ -151,8 +216,15 @@ class BaseConverter(object):
         @returns:
         """
 
+    def get_mask(self):
+        """
+        @returns:
+        """
+        return None
+
 class _StringConverter(BaseConverter):
     type = str
+    name = _('String')
 
     def as_string(self, value, format=None):
         if format is None:
@@ -166,6 +238,7 @@ converter.add(_StringConverter)
 
 class _UnicodeConverter(BaseConverter):
     type = unicode
+    name = _('Unicode')
 
     def as_string(self, value, format=None):
         if format is None:
@@ -179,6 +252,8 @@ converter.add(_UnicodeConverter)
 
 class _IntConverter(BaseConverter):
     type = int
+    name = _('Integer')
+    align = Alignment.RIGHT
 
     def as_string(self, value, format=None):
         """Convert a float to a string"""
@@ -195,7 +270,7 @@ class _IntConverter(BaseConverter):
         if value == '':
             return ValueUnset
 
-        conv = locale.localeconv()
+        conv = get_localeconv()
         thousands_sep = conv["thousands_sep"]
         # Remove all thousand separators, so int() won't barf at us
         if thousands_sep and thousands_sep in value:
@@ -205,16 +280,18 @@ class _IntConverter(BaseConverter):
             return self.type(value)
         except ValueError:
             raise ValidationError(
-                "%s could not be converted to an integer" % value)
+                _("%s could not be converted to an integer") % value)
 
 converter.add(_IntConverter)
 
 class _LongConverter(_IntConverter):
     type = long
+    name = _('Long')
 converter.add(_LongConverter)
 
 class _BoolConverter(BaseConverter):
     type = bool
+    name = _('Boolean')
 
     def as_string(self, value, format=None):
         return str(value)
@@ -229,12 +306,14 @@ class _BoolConverter(BaseConverter):
         elif value.upper() in ('FALSE', '0'):
             return False
 
-        return ValidationError("'%s' can not be converted to a boolean" % value)
+        return ValidationError(_("'%s' can not be converted to a boolean") % value)
 
 converter.add(_BoolConverter)
 
 class _FloatConverter(BaseConverter):
     type = float
+    name = _('Float')
+    align = Alignment.RIGHT
 
     def as_string(self, value, format=None):
         """Convert a float to a string"""
@@ -261,7 +340,7 @@ class _FloatConverter(BaseConverter):
             # When format is '%g', if value is an integer, the result
             # will also be formated as an integer, so we add a '.0'
 
-            conv = locale.localeconv()
+            conv = get_localeconv()
             as_str +=  conv.get('decimal_point') + '0'
 
         return as_str
@@ -276,7 +355,7 @@ class _FloatConverter(BaseConverter):
         try:
             retval = float(value)
         except ValueError:
-            raise ValidationError("This field requires a number, not %r" %
+            raise ValidationError(_("This field requires a number, not %r") %
                                   value)
 
         return retval
@@ -285,6 +364,8 @@ converter.add(_FloatConverter)
 
 class _DecimalConverter(_FloatConverter):
     type = Decimal
+    name = _('Decimal')
+    align = Alignment.RIGHT
     def from_string(self, value):
         if value == '':
             return ValueUnset
@@ -294,13 +375,41 @@ class _DecimalConverter(_FloatConverter):
         try:
             retval = Decimal(value)
         except InvalidOperation:
-            raise ValidationError("This field requires a number, not %r" %
+            raise ValidationError(_("This field requires a number, not %r") %
                                   value)
 
         return retval
 
 converter.add(_DecimalConverter)
 
+# Constants for use with win32
+LOCALE_SSHORTDATE = 31
+LOCALE_STIMEFORMAT = 4099
+DATE_REPLACEMENTS_WIN32 = [
+    (re.compile('HH?'), '%H'),
+    (re.compile('hh?'), '%I'),
+    (re.compile('mm?'), '%M'),
+    (re.compile('ss?'), '%S'),
+    (re.compile('tt?'), '%p'),
+    (re.compile('dd?'), '%d'),
+    (re.compile('MM?'), '%m'),
+    (re.compile('yyyyy?'), '%Y'),
+    (re.compile('yy?'), '%y')
+    ]
+
+DATE_MASK_TABLE = {
+    '%m': '00',
+    '%y': '00',
+    '%d': '00',
+    '%Y': '0000',
+    '%H': '00',
+    '%M': '00',
+    '%S': '00',
+    '%T': '00:00:00',
+    # FIXME: locale specific
+    '%r': '00:00:00 LL',
+    }
+
 class _BaseDateTimeConverter(BaseConverter):
     """
     Abstract class for converting datatime objects to and from strings
@@ -308,11 +417,15 @@ class _BaseDateTimeConverter(BaseConverter):
     @cvar lang_constant:
     """
     date_format = None
+    align = Alignment.RIGHT
 
     def __init__(self):
         self._keep_am_pm = False
         self._keep_seconds = False
 
+    def get_lang_constant_win32(self):
+        raise NotImplementedError
+
     def get_lang_constant(self):
         # This is a method and not a class variable since it does not
         # exist on all supported platforms, eg win32
@@ -337,7 +450,16 @@ class _BaseDateTimeConverter(BaseConverter):
 
     def get_format(self):
         if sys.platform == 'win32':
-            format = self.date_format
+            values = []
+            for constant in self.get_lang_constant_win32():
+                value = GetLocaleInfo(constant)
+                values.append(value)
+            format = " ".join(values)
+
+            # Now replace them to strftime like masks so the logic
+            # below still applies
+            for pattern, replacement in DATE_REPLACEMENTS_WIN32:
+                format = pattern.subn(replacement, format)[0]
         else:
             format = locale.nl_langinfo(self.get_lang_constant())
 
@@ -362,6 +484,13 @@ class _BaseDateTimeConverter(BaseConverter):
 
         return format
 
+    def get_mask(self):
+        mask = self.get_format()
+        for format_char, mask_char in DATE_MASK_TABLE.items():
+            mask = mask.replace(format_char, mask_char)
+
+        return mask
+
     def as_string(self, value, format=None):
         "Convert a date to a string"
         if format is None:
@@ -390,12 +519,16 @@ class _BaseDateTimeConverter(BaseConverter):
             return self.from_dateinfo(dateinfo)
         except ValueError:
             raise ValidationError(
-                'This field requires a date of the format "%s" and '
-                'not "%s"' % (format, value))
+                _('This field requires a date of the format "%s" and '
+                  'not "%s"') % (format, value))
 
 class _TimeConverter(_BaseDateTimeConverter):
     type = datetime.time
+    name = _('Time')
     date_format = '%X'
+    def get_lang_constant_win32(self):
+        return [LOCALE_STIMEFORMAT]
+
     def get_lang_constant(self):
         return locale.T_FMT
 
@@ -406,7 +539,11 @@ converter.add(_TimeConverter)
 
 class _DateTimeConverter(_BaseDateTimeConverter):
     type = datetime.datetime
+    name = _('Date and Time')
     date_format = '%c'
+    def get_lang_constant_win32(self):
+        return [LOCALE_SSHORTDATE, LOCALE_STIMEFORMAT]
+
     def get_lang_constant(self):
         return locale.D_T_FMT
 
@@ -417,7 +554,11 @@ converter.add(_DateTimeConverter)
 
 class _DateConverter(_BaseDateTimeConverter):
     type = datetime.date
+    name = _('Date')
     date_format = '%x'
+    def get_lang_constant_win32(self):
+        return [LOCALE_SSHORTDATE]
+
     def get_lang_constant(self):
         return locale.D_FMT
 
@@ -428,177 +569,42 @@ converter.add(_DateConverter)
 
 class _ObjectConverter(BaseConverter):
     type = object
+    name = _('Object')
 
     as_string = None
     from_string = None
 converter.add(_ObjectConverter)
 
-class currency(Decimal):
-    """
-    A datatype representing currency, used together with the list and
-    the framework
-    """
-    _converter = converter.get_converter(Decimal)
-
-    def __new__(cls, value):
-        """
-        @param value: value to convert
-        @type value: string or number
-        """
-        if isinstance(value, str):
-            conv = locale.localeconv()
-            currency_symbol = conv.get('currency_symbol')
-            text = value.strip(currency_symbol)
-            value = currency._converter.from_string(text)
-            if value == ValueUnset:
-                raise InvalidOperation
-        elif isinstance(value, float):
-            print ('Warning: losing precision converting float %r to currency'
-                   % value)
-            value = str(value)
-        elif not isinstance(value, (int, long, Decimal)):
-            raise TypeError(
-                "cannot convert %r of type %s to a currency" % (
-                value, type(value)))
-
-        return Decimal.__new__(cls, value)
-
-    def format(self, symbol=True, precision=None):
-        value = Decimal(self)
-
-        conv = locale.localeconv()
-
-        # Grouping (eg thousand separator) of integer part
-        groups = conv.get('mon_grouping', [])[:]
-        groups.reverse()
-        if groups:
-            group = groups.pop()
-        else:
-            group = 3
-
-        intparts = []
-
-        # We're iterating over every character in the integer part
-        # make sure to remove the negative sign, it'll be added later
-        intpart = str(int(abs(value)))
-
-        while True:
-            if not intpart:
-                break
-
-            s = intpart[-group:]
-            intparts.insert(0, s)
-            intpart = intpart[:-group]
-            if not groups:
-                continue
-
-            last = groups.pop()
-            # if 0 reuse last one, see struct lconv in locale.h
-            if last != 0:
-                group = last
-
-        # Add the sign, and the list of decmial parts, which now are grouped
-        # properly and can be joined by mon_thousand_sep
-        if value > 0:
-            sign = conv.get('positive_sign', '')
-        elif value < 0:
-            sign = conv.get('negative_sign', '-')
-        else:
-            sign = ''
-        currency = sign + conv.get('mon_thousands_sep', '.').join(intparts)
-
-        # Only add decimal part if it has one, is this correct?
-        if precision is not None or value % 1 != 0:
-            # Pythons string formatting can't handle %.127f
-            # 127 is the default value from glibc/python
-            if precision:
-                frac_digits = precision
-            else:
-                frac_digits = conv.get('frac_digits', 2)
-                if frac_digits == 127:
-                    frac_digits = 2
-
-            format = '%%.%sf' % frac_digits
-            dec_part = (format % value)[-frac_digits:]
-
-            mon_decimal_point = conv.get('mon_decimal_point', '.')
-            currency += mon_decimal_point + dec_part
-
-        # If requested include currency symbol
-        currency_symbol = conv.get('currency_symbol', '')
-        if currency_symbol and symbol:
-            if value > 0:
-                cs_precedes = conv.get('p_cs_precedes', 1)
-                sep_by_space = conv.get('p_sep_by_space', 1)
-            else:
-                cs_precedes = conv.get('n_cs_precedes', 1)
-                sep_by_space = conv.get('n_sep_by_space', 1)
-
-            # Patching glibc's output
-            # See http://sources.redhat.com/bugzilla/show_bug.cgi?id=1294
-            current_locale = locale.getlocale(locale.LC_MONETARY)
-            if current_locale[0] == 'pt_BR':
-                cs_precedes = 1
-                sep_by_space = 0
-
-            if sep_by_space:
-                space = ' '
-            else:
-                space = ''
-            if cs_precedes:
-                currency = currency_symbol + space + currency
-            else:
-                currency = currency + space + currency_symbol
-
-        return currency
-
-    def __repr__(self):
-        return '<currency %s> ' % self.format()
-
-class _CurrencyConverter(_DecimalConverter):
-    type = currency
-
-    def __init__(self):
-        self.symbol = True
-        self.precision = 2
-
-    def as_string(self, value, format=None):
-        if not isinstance(value, currency):
-            try:
-                value = currency(value)
-            except ValueError:
-                raise ValidationError(
-                    "%s can not be converted to a currency" % value)
-
-        return value.format(self.symbol, self.precision)
-
-    def from_string(self, value):
-        if value == '':
-            return ValueUnset
-        try:
-            return currency(value)
-        except (ValueError, InvalidOperation):
-            raise ValidationError(
-                "%s can not be converted to a currency" % value)
-
-converter.add(_CurrencyConverter)
-
 def lformat(format, value):
     """Like locale.format but with grouping enabled"""
     return locale.format(format, value, 1)
 
-def format_price(value, symbol=True, precision=None):
-    """
-    Formats a price according to the current locales monetary
-    settings
+def get_localeconv():
+    conv = locale.localeconv()
 
-    @param value: number
-    @param symbol: whether to include the currency symbol
-    """
+    monetary_locale = locale.getlocale(locale.LC_MONETARY)
+    numeric_locale = locale.getlocale(locale.LC_NUMERIC)
+    # Patching glibc's output
+    # See http://sources.redhat.com/bugzilla/show_bug.cgi?id=1294
+    if monetary_locale[0] == 'pt_BR':
+        conv['p_cs_precedes'] = 1
+        conv['p_sep_by_space'] = 1
 
-    return currency(value).format(symbol, precision)
+    # Since locale 'C' doesn't have any information on monetary and numeric
+    # locale, use default en_US, so we can have formated numbers
+    if not monetary_locale[0]:
+        conv["negative_sign"] = '-'
+        conv["currency_symbol"] = '$'
+        conv['mon_thousands_sep'] = ''
+        conv['mon_decimal_point'] = '.'
+        conv['p_sep_by_space'] = 0
 
-def filter_locale(value):
+    if not numeric_locale[0]:
+        conv['decimal_point'] = '.'
+
+    return conv
+
+def filter_locale(value, monetary=False):
     """
     Removes the locale specific data from the value string.
     Currently we only remove the thousands separator and
@@ -606,29 +612,22 @@ def filter_locale(value):
     The returned value of this function can safely be passed to float()
 
     @param value: value to convert
+    @param monetary: if we should treat it as monetary data or not
     @returns: the value without locale specific data
     """
 
-    conv = locale.localeconv()
+    def _filter_thousands_sep(value, thousands_sep):
+        if not thousands_sep:
+            return value
 
-    # Check so we only have one decimal point
-    decimal_point = conv["decimal_point"]
-    decimal_points = value.count(decimal_point)
-    if decimal_points > 1:
-        raise ValidationError(
-            'You have more than one decimal point ("%s") '
-            ' in your number "%s"' % (decimal_point, value))
-
-    thousands_sep = conv["thousands_sep"]
-    if thousands_sep:
         # Check so we don't have any thousand separators to the right
         # of the decimal point
         th_sep_count = value.count(thousands_sep)
         if th_sep_count and decimal_points:
             decimal_point_pos = value.index(decimal_point)
             if thousands_sep in value[decimal_point_pos+1:]:
-                raise ValidationError("You have a thousand separator to the "
-                                      "right of the decimal point")
+                raise ValidationError(_("You have a thousand separator to the "
+                                        "right of the decimal point"))
             check_value = value[:decimal_point_pos]
         else:
             check_value = value
@@ -639,18 +638,48 @@ def filter_locale(value):
 
         # First part is a special case, It can be 1, 2 or 3
         if 3 > len(parts[0]) < 1:
-            raise ValidationError("Inproperly placed thousands separator")
+            raise ValidationError(_("Inproperly placed thousands separator"))
 
         # Middle parts should have a length of 3
         for part in parts[1:]:
             if len(part) != 3:
-                raise ValidationError("Inproperly placed thousand "
-                                      "separators")
+                raise ValidationError(_("Inproperly placed thousand "
+                                        "separators: %r" % (parts,)))
 
         # Remove all thousand separators
-        value = value.replace(thousands_sep, '')
+        return value.replace(thousands_sep, '')
+
+    conv = get_localeconv()
+
+    # Check so we only have one decimal point
+    if monetary:
+        decimal_point = conv["mon_decimal_point"]
+    else:
+        decimal_point = conv["decimal_point"]
+
+    if decimal_point != '':
+        decimal_points = value.count(decimal_point)
+        if decimal_points > 1:
+            raise ValidationError(
+                _('You have more than one decimal point ("%s") '
+                  ' in your number "%s"' % (decimal_point, value)))
+
+    if monetary:
+        sep = conv["mon_thousands_sep"]
+    else:
+        sep = conv["thousands_sep"]
+
+    if sep and sep in value:
+        value = _filter_thousands_sep(value, sep)
 
     # Replace all decimal points with .
-    if decimal_point != '.':
+    if decimal_point != '.' and decimal_point != '':
         value = value.replace(decimal_point, '.')
     return value
+
+# FIXME: Get rid of this
+from kiwi.currency import currency, format_price
+
+# Pyflakes
+currency
+format_price
