@@ -100,6 +100,12 @@ class Environment:
 
     def add_resources(self, **kwargs):
         for resource, path in kwargs.items():
+            if resource == 'locale':
+                try:
+                    self.add_resource(resource, path)
+	        except EnvironmentError:
+                    continue
+            
             self.add_resource(resource, path)
 
     def find_resource(self, resource, name):
@@ -117,7 +123,7 @@ class Environment:
         for path in resource_paths + [scriptdir, ""]:
             for ext in exts:
                 filename = os.path.join(self._root, path, "".join((name, ext)))
-                if os.path.exists(filename):
+                if os.path.exists(filename) and os.path.isfile(filename):
                     return filename
 
         raise EnvironmentError("Could not find %s resource: %s" % (
@@ -284,10 +290,15 @@ class Library(Environment):
             return
         directory = gettext.bindtextdomain(domain, localedir[0])
         self._check_translation(domain, directory)
+        # For libglade, but only on non-win32 systems
+        if hasattr(locale, 'bindtextdomain'):
+            locale.bindtextdomain(domain, localedir[0])
 
         # Gtk+ only supports utf-8, it makes no sense to support
         # other encodings in kiwi it self
-        gettext.bind_textdomain_codeset(domain, 'utf-8')
+        # This is not present in Python 2.3
+        if hasattr(gettext, 'bind_textdomain_codeset'):
+            gettext.bind_textdomain_codeset(domain, 'utf-8')
 
     def set_application_domain(self, domain):
         """
@@ -295,6 +306,9 @@ class Library(Environment):
         @param domain: the domain
         """
         gettext.textdomain(domain)
+        # For libglade, but only on non-win32 systems
+        if hasattr(locale, 'textdomain'):
+            locale.textdomain(domain)
 
     def add_global_resource(self, resource, path):
         """Convenience method to add a global resource.
