@@ -72,6 +72,80 @@ class IValidatableProxyWidget(IProxyWidget):
     def validate(force=False):
         pass
 
+class IEasyCombo(Interface):
+
+    def prefill(itemdata, sort=False):
+        """Fills the Combo with listitems corresponding to the itemdata
+        provided.
+
+        Parameters:
+          - itemdata is a list of strings or tuples, each item corresponding
+            to a listitem. The simple list format is as follows::
+
+            >>> [ label0, label1, label2 ]
+
+            If you require a data item to be specified for each item, use a
+            2-item tuple for each element. The format is as follows::
+
+            >>> [ ( label0, data0 ), (label1, data1), ... ]
+
+          - Sort is a boolean that specifies if the list is to be sorted by
+            label or not. By default it is not sorted
+        """
+
+    def append_item(label, data=None):
+        """ Adds a single item to the Combo.
+        @param label: a string with the text to be added
+        @param data: the data to be associated with that item
+        """
+
+    def clear():
+        """Removes all items from the widget"""
+
+    def select(data):
+        """
+        @param obj: data or text to select
+        """
+
+    def select_item_by_position(position):
+        """
+        Selects an item in the combo from a integer where 0
+        represents the first item.
+        @param position: an integer
+        """
+
+    def select_item_by_label(text):
+        """
+        @param text: text to select
+        """
+
+    def select_item_by_data(data):
+        """
+        @param data: object to select
+        """
+
+    def get_selected_label():
+        """
+        @returns: the label of the currently selected item
+        """
+
+    def get_selected_data():
+        """
+        @returns: the data of the currently selected item
+        """
+
+    def get_selected():
+        """
+        @returns: selected text or item or None if nothing
+          is selected
+        """
+
+    def get_model_strings():
+        pass
+
+    def get_model_items():
+        pass
+
 class AbstractGladeAdaptor(Interface):
     """Abstract class that define the functionality an class that handle
     glade files should provide."""
